#!/bin/ksh
#\
exec wish "$0" "$@"

set tzdir {}

proc readtz {fname} {
set ttinfo {}
set times {}
set ttimes {}
set tz {}
set leap {}
set dst {}

set fid [open $fname RDONLY]
set btime [read $fid [file size $fname]]
close $fid
binary scan $btime a24 head
binary scan $btime a24IIIII head isstdcnt leapcnt timecnt typecnt charcnt
#puts "$isstdcnt $leapcnt $timecnt $typecnt $charcnt"

for {set i 0} {$i<$timecnt} {incr i} {
	set fmt @[expr 44 + 4*$i]I
	binary scan $btime $fmt var
	lappend times $var
}
#puts $times

for {set i 0} {$i<$timecnt} {incr i} {
	set fmt @[expr 44 + 4*$timecnt + $i]c
	binary scan $btime $fmt var
	lappend ttimes $var
}
#puts $ttimes

for {set i 0} {$i<$typecnt} {incr i} {
	set fmt @[expr 44 + 4*$timecnt + $timecnt + 6*$i]Icc
	binary scan $btime $fmt offset isdt abbrind
	lappend ttinfo [list [expr $offset / 60] $isdt $abbrind]
}
#puts $ttinfo

binary scan $btime @[expr 44 + 4*$timecnt + $timecnt + 6*$typecnt]a* tz
#puts $tz

for {set i 0} {$i<$leapcnt} {incr i} {
	set fmt @[expr 44 + 4*$timecnt + $timecnt + 6*$typecnt + $charcnt]II
	binary scan $btime $fmt ltime nleap
	lappend leap [list $ltime $nleap]
}
#puts "$leap"
for {set i 0} {$i<$isstdcnt} {incr i} {
	set fmt @[expr 44 + 4*$timecnt + $timecnt + 6*$typecnt + $charcnt + 8*$leapcnt]c
	binary scan $btime $fmt var
	lappend isstd $var
}
#puts $isstd

set curyear [clock format [clock seconds] -format %Y]
for {set i 0} {$i<$timecnt} {incr i} {
		if {[clock format [lindex $times $i] -format %Y]==$curyear} {
			lappend dst $i
#			puts "[clock format [lindex $times $i]] $dst"
		}
}
#puts "[clock format [lindex $times [expr $i + 1]]]"
#puts "$i entries"
#puts "dst   : \n\t$dst"
#puts "ttinfo: \n\t$ttinfo"
#puts "ttimes: \n\t$ttimes"
#puts "times :  \n\t$times"

set ttoff 0
if $timecnt {
	set ttoff [lindex \
			[lindex $ttinfo \
				[lindex $ttimes \
					[expr {$dst=={}?"end":[lindex $dst 0]}] \
				] \
			] \
		0]
}
#puts "setting $ttoff"
return $ttoff
}

proc build_tzlist {w dir} {
set tzlist {}
global zoneinfo
	set flist [glob -nocomplain $zoneinfo/$dir/*]
	$w delete 0 end
	foreach f $flist {if {[file type $f]=="file"} {lappend tzlist $f}}
	foreach f [lsort $tzlist] {$w insert end [file tail $f]}
}

proc _timezone {w} {
global labelbgcolor labelfgcolor labelfont zoneinfo
global buttonbgcolor buttonfgcolor buttonfont
global tzdir

if [winfo exists $w] return
toplevel $w
wm title $w "Timezones Editor"
wm iconname $w "Timezones Editor"
grab $w

frame $w.tzmain -bg $labelbgcolor
frame $w.tzbuttons -bg $labelbgcolor -bd 2 -relief ridge

menubutton $w.tzmain.mb -direction below -indicatoron yes -menu \
	$w.tzmain.mb.m -text "." -bg $labelbgcolor -font $labelfont -fg $labelfgcolor -relief raised
menu $w.tzmain.mb.m -tearoff 0 -bg $labelbgcolor -font $labelfont -fg $labelfgcolor
foreach f [glob -nocomplain $zoneinfo/*] {
	if {[file type $f]=="directory"} {
		$w.tzmain.mb.m add command -label [file tail $f] \
		-command "set tzdir [file tail $f];
			  $w.tzmain.mb configure -text [file tail $f];
			  build_tzlist $w.tzmain.lb [file tail $f]"
	}
}
scrollbar $w.tzmain.sb -command "$w.tzmain.lb yview" -width 10 -bg $labelbgcolor
listbox $w.tzmain.lb -font $labelfont -selectmode single \
	-yscroll "$w.tzmain.sb set" -setgrid 1

build_tzlist $w.tzmain.lb .

button $w.tzbuttons.dismiss -text Dismiss -command "destroy $w" -bd 1 \
	-bg $buttonbgcolor -font $buttonfont -fg $buttonfgcolor
#button $w.tzbuttons.save -text Save -command "destroy $w" -bd 1\
#	-bg $buttonbgcolor -font $buttonfont -fg $buttonfgcolor
button $w.tzbuttons.add -text Add -command "add_tz $w.tzmain" -bd 1 \
	-bg $buttonbgcolor -font $buttonfont -fg $buttonfgcolor

frame $w.tzmain.l -bd 5 -bg $labelbgcolor
label $w.tzmain.l.title -text "Current Timezones" -justify center \
	-font lucidasanstypewriter-bold-14 -fg $labelfgcolor -bg $labelbgcolor -relief raised -bd 1
pack $w.tzmain.l $w.tzmain.l.title -expand yes -fill x -anchor n

addall_tz $w.tzmain.l

pack $w.tzmain.mb -expand yes -fill x
pack $w.tzmain.lb $w.tzmain.sb -side left -fill y
pack $w.tzmain.l -side left -fill x -expand yes -fill both
pack $w.tzmain $w.tzbuttons -fill x
pack $w.tzbuttons.add $w.tzbuttons.dismiss -side left -expand yes -fill x

}

proc addall_tz {w} {
global labelbgcolor labelfgcolor labelfont bitmapcolor timezonelist

foreach win [winfo children $w] {if {$win!="$w.title"} {destroy $win}}

if [info exists timezonelist] {
	set i 0
	foreach t $timezonelist {
		if {$i >= 8} return
		frame $w.f$i -bg $labelbgcolor
		label $w.f$i.e -text "[lindex $t 1]" \
			-bg $labelbgcolor -fg $labelfgcolor -font $labelfont
		label $w.f$i.l -text "[to_offset $t]" -justify left -width 10 \
			-bg $labelbgcolor -fg $labelfgcolor -font $labelfont \
			-relief flat -bd 0
		label $w.f$i.b -text X \
			-bg $labelbgcolor -fg $bitmapcolor -font $labelfont \
			-relief flat -bd 0
		bind $w.f$i.b <Double-Button-1> "delete_tz $w $i"
		bind $w.f$i.l <Double-Button-1> "toggle_home $w $i"
		pack $w.f$i -fill x
		pack $w.f$i.e -side left -fill x -expand yes
		pack $w.f$i.l -side left -fill x -expand yes
		pack $w.f$i.b -side left -expand no -anchor e -ipadx 0 -ipady 0
		incr i
	}
}
}	

proc to_offset {t} {
	set o [lindex $t 2]
	
	if {$o=="home"} {return $o}

	if {$o=="away"} {set o [lindex $t 3]}

	set o [format "%s%#d:%#02dh" [expr {$o>=0?"+":"-"}] \
		[expr abs($o)/60] [expr abs($o)%60]]
	return $o
}

proc toggle_home {w i} {
global timezonelist
set newtimezonelist {}

foreach tz $timezonelist {
	if {[lindex $tz 2]=="home"} {
		set tz "timezone \"[lindex $tz 1]\" [lrange $tz 3 e]"
	}
	lappend newtimezonelist $tz
}
set timezonelist $newtimezonelist
unset newtimezonelist
set tz [lindex $timezonelist $i] 
set timezonelist [lreplace $timezonelist $i $i \
	"timezone \"[lindex $tz 1]\" home [lrange $tz 2 e]" ]
addall_tz $w
}

proc delete_tz {w i} {
global timezonelist
if {[tk_messageBox -icon question -type yesno \
	-title "Are You Sure ?" \
	-message "Are You Sure"]=="no"} return
set timezonelist [lreplace $timezonelist $i $i]
addall_tz $w
}

proc add_tz {w} {
global timezonelist zoneinfo tzdir

if {[$w.lb curselection]=={}} {
	tk_messageBox -icon error -type ok -title "Error" \
		-message "No Timezone Selected !"
	return
}
set tz [$w.lb get [$w.lb curselection]]

if [info exists timezonelist] {
	if {[llength $timezonelist]>=8} {
		tk_messageBox -icon error -type ok -title "Error" \
			-message "Only 8 Timezones Allowed !"
		return
	}

	foreach t $timezonelist {
		if {[lindex $t 1]=="$tz"} {
			tk_messageBox -icon error -type ok -title "Error" \
				-message "Timezone Already Exists !"
			return
		}
	}
}

set ttoff [readtz $zoneinfo/$tzdir/$tz]
lappend timezonelist "timezone \"$tz\" $ttoff 0 0 0 0 0 0 0 0 0 0"
addall_tz $w.l
}
